# -*- coding: utf-8 -*-

from odoo import fields, models
from odoo.tools import consteq
import hashlib
import hmac

import logging
_logger = logging.getLogger(__name__)


class SaleOrderLine(models.Model):
    _inherit = 'sale.order.line'

    certif_available = fields.Integer()
    certif_channel_id = fields.Many2one('slide.channel')

    # public to be available in email template but need to be called in sudo
    def get_certif_link(self):
        assert(self.env.is_system())
        secret = self.env['ir.config_parameter'].get_param('database.secret')
        return hmac.new(secret.encode('utf-8'), str("SaleOrderLine-%d" % self.id).encode('utf-8'), hashlib.sha256).hexdigest()

    def _check_certif_link(self, token):
        # need to be called in sudo
        return consteq(token, self.get_certif_link())


class SaleOrder(models.Model):
    _inherit = 'sale.order'

    def _action_confirm(self):
        # call super, but don't add member really

        if self.website_id:
            # default behavior
            result = super(SaleOrder, self)._action_confirm()

        else:
            self = self.with_context(no_add_member=True)
            result = super(SaleOrder, self)._action_confirm()

            products = self.order_line.mapped('product_id')
            related_channels = self.env['slide.channel'].search([('enroll', '=', 'payment'), ('product_id', 'in', products.ids)])
            if related_channels:
                for sol in self.order_line:
                    if sol.product_id in related_channels.mapped('product_id'):
                        related_channel = related_channels.filtered(lambda chan: chan.product_id == sol.product_id)
                        assert(len(related_channel) == 1)
                        sol.write({
                            'certif_available': sol.product_uom_qty,
                            'certif_channel_id': related_channel.id,
                        })
                        log = "%d certif assigned to SaleOrder %d (line %d) for channel: %s (%d)" % (sol.product_uom_qty, sol.order_id.id, sol.id, related_channel.name, related_channel.id)
                        sol.order_id.message_post(body=log, message_type='notification')
                        _logger.info(log)
        return result


class Channel(models.Model):

    _inherit = 'slide.channel'

    def _action_add_members(self, target_partners, **member_values):

        # Internally, we don't handle the new member on payment,
        # but with a custom controller to allow multi user in once order
        if self.env.context.get('no_add_member'):
            return self.env['slide.channel.partner'].sudo()

        return super(Channel, self)._action_add_members(target_partners, **member_values)
